# -*- coding: utf-8 -*-
'''
Created on Sun Dec  8 14:49:19 2024

@author: mstep
'''

import tkinter as tk

class DragFrame(tk.Frame):
    def __init__(self, frame, *args, **kwargs):
        tk.Frame.__init__(self, frame, *args, **kwargs)

        # Canvas.
        self.canvas = tk.Canvas(self)
        self.canvas.pack(expand=True, fill=tk.BOTH)

        # Frame.
        self.frame = tk.Frame(self.canvas)
        self.frame_window = self.canvas.create_window(0, 0,
            window=self.frame, anchor=tk.NW)

        # Initially we are not moving.
        self.start_x = None

    def configure_frame(self):
        self.frame.update_idletasks()
        self.frame.place(x=0, y=0,
            width=self.frame.winfo_reqwidth(),
            height=self.frame.winfo_reqheight())

        # Track Button-1 down, Button-1 motion, and Button-1 up.
        for child in widget_and_descendants(self.canvas):
            child.bind('<Button-1>', self.mouse_down)
            child.bind('<B1-Motion>', self.mouse_move)
            child.bind('<ButtonRelease-1>', self.mouse_up)

    def mouse_down(self, event):
        '''Record the starting info.'''
        # Save the mouse's position with respect to the root window.
        self.start_x = event.x_root
        self.start_y = event.y_root

        # Save the frame's current location.
        self.frame_x = self.frame.winfo_x()
        self.frame_y = self.frame.winfo_y()

        # Calculate maximum X and Y coordinates.
        self.min_x = self.canvas.winfo_width() - self.frame.winfo_width()
        self.min_y = self.canvas.winfo_height() - self.frame.winfo_height()

    def mouse_move(self, event):
        # Ignore extraneous events.
        if self.start_x is None: return

        # Move the frame.
        x = self.frame_x + (event.x_root - self.start_x)
        y = self.frame_y + (event.y_root - self.start_y)
        if x < self.min_x: x = self.min_x
        if y < self.min_y: y = self.min_y
        if x > 0: x = 0
        if y > 0: y = 0
        self.frame.place(x=x, y=y)

    def mouse_up(self, event):
        self.start_x = None

def widget_and_descendants(parent):
    '''Return this widget and all of its descendants.'''
    results = [parent]
    for child in parent.winfo_children():
        results += widget_and_descendants(child)
    return results

#%%
import tkinter as tk
from PIL import Image, ImageTk

class DragFrameApp:
    def __init__(self):
        # Make the main interface.
        self.window = tk.Tk()
        self.window.title('drag_frame')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('400x300')

        # Make a DragFrame.
        drag_frame = DragFrame(self.window)
        drag_frame.pack(side=tk.TOP, fill=tk.BOTH, expand=True,
                            padx=3, pady=3)

        # Save the frame so we can put things in it.
        my_frame = drag_frame.frame
        my_frame.config(cursor='hand2')

        ###############################
        # Put some things in the frame.
        label = tk.Label(my_frame, text='Who\ncould\nask\nfor\nmore?',
                         font=('Calibri', 20, 'bold'))
        label.pack(side=tk.LEFT, expand=True, fill=tk.Y)

        path = 'robot_eating_ice_cream.jpg'
        image = Image.open(path)
        self.tk_image = ImageTk.PhotoImage(image)
        image_label = tk.Label(my_frame, image=self.tk_image)
        image_label.pack(side=tk.LEFT)

        # Configure the frame. (Do this after adding all contained widgets.)
        drag_frame.configure_frame()

        # Display the window.
        self.window.mainloop()

    def kill_callback(self):
        self.window.destroy()

#%%

DragFrameApp()
print('Done')
